/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;
in vec3 in_Normal;
in vec2 in_Bones;

out vec2 v_TexCoord;
out vec4 v_ShadowMapCoord;
out vec3 v_WorldNormal;
out vec3 v_WorldPosition;
out vec4 v_ProjectedVertex;

#include MaterialSystem/ShaderLib/Uniforms_Vertex.sp
#include MaterialSystem/ShaderLib/Projections.sp

#if defined( SUPPORTED_VERTEX_TEXTURE_FETCH )
#include MaterialSystem/ShaderLib/QuaternionLib.sp
#include MaterialSystem/ShaderLib/Skinning.sp
#endif

void main()
{
#if defined( SUPPORTED_VERTEX_TEXTURE_FETCH )
   vec4 Position = ( u_KeyFrame.w == 1.0 ) ? SkinVertex() : in_Vertex;
#else
   vec4 Position = in_Vertex;
#endif

   gl_Position = in_ModelViewProjectionMatrix * Position;

   v_TexCoord = in_TexCoord.xy;
	v_ShadowMapCoord = in_ShadowMatrix[0] * in_ModelMatrix * Position;
	v_WorldNormal = ( in_ModelMatrix * vec4( in_Normal, 0.0 ) ).xyz;
	v_WorldPosition = ( in_ModelMatrix * Position ).xyz;
	v_ProjectedVertex = GetProjScaleBiasMat() * in_ModelViewProjectionMatrix * Position;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 v_TexCoord;
in vec4 v_ShadowMapCoord;
in vec3 v_WorldNormal;
in vec3 v_WorldPosition;
in vec4 v_ProjectedVertex;

out vec4 out_FragColor;

#include MaterialSystem/ShaderLib/Uniforms_Fragment.sp

#include MaterialSystem/ShaderLib/Pack.sp
#include MaterialSystem/ShaderLib/ShadowUtils.sp
#include MaterialSystem/ShaderLib/LightingUtils.sp
#include MaterialSystem/ShaderLib/Lighting_Lambert.sp
#include MaterialSystem/ShaderLib/Lighting_OrenNayar.sp
#include MaterialSystem/ShaderLib/Lighting_CookTorrance.sp
#include MaterialSystem/ShaderLib/Shadows.sp

void main()
{
	vec4 Kd = texture( Texture2, v_TexCoord ) + u_DiffuseColor;
	vec4 Ks = vec4( 0.0 );

	// alpha test
	if ( u_MaterialAlphaTest.x > 0.0 )
	{
		if ( Kd.a < u_MaterialAlphaTest.x ) discard;
	}

 	vec4 FinalColor = Kd;

	float LightType = u_LightAttenuation.w;
	float ShadingType = u_MaterialShading.x;

	const float LIGHT_TYPE_NONE        = 0.0;
	const float LIGHT_TYPE_DIRECTIONAL = 1.0;
	const float LIGHT_TYPE_SPOT        = 2.0;
	const float LIGHT_TYPE_POINT       = 3.0;

	const float SHADING_AMBIENT      = 0.0;
	const float SHADING_LAMBERT      = 1.0;
	const float SHADING_ORENNAYAR    = 2.0;
	const float SHADING_COOKTORRANCE = 3.0;

	vec3 WorldNormal = normalize(v_WorldNormal);

	if ( LightType == LIGHT_TYPE_DIRECTIONAL )
	{
		if ( ShadingType == SHADING_AMBIENT )
		{
			FinalColor = vec4( 0.0 );
		}
		else if ( ShadingType == SHADING_ORENNAYAR )
		{
			FinalColor = ComputeDirectionalLight_OrenNayar( WorldNormal, Kd, Ks );
		}
		else
		{
			FinalColor = ComputeDirectionalLight_Lambert( WorldNormal, Kd, Ks );
		}
		FinalColor *= ComputeDirectionalLightShadow();
	}
	else if ( LightType == LIGHT_TYPE_SPOT )
	{
		if ( ShadingType == SHADING_AMBIENT )
		{
			FinalColor = vec4( 0.0 );
		}
		else if ( ShadingType == SHADING_ORENNAYAR )
		{
			FinalColor = ComputeSpotLight_OrenNayar( WorldNormal, Kd, Ks );
		}
		else
		{
			FinalColor = ComputeSpotLight_Lambert( WorldNormal, Kd, Ks );
		}
		FinalColor *= ComputeDirectionalLightShadow();
	}
	else if ( LightType == LIGHT_TYPE_POINT )
	{
		if ( ShadingType == SHADING_AMBIENT )
		{
			FinalColor = vec4( 0.0 );
		}
		else if ( ShadingType == SHADING_ORENNAYAR )
		{
			FinalColor = ComputePointLight_OrenNayar( WorldNormal, Kd, Ks );
		}
		else
		{
			FinalColor = ComputePointLight_Lambert( WorldNormal, Kd, Ks );
		}
		FinalColor *= ComputePointLightShadow();
	}

	FinalColor += Kd * u_LightAmbientColor;

	float Transparency = u_MaterialParams.x;

	FinalColor.a = 1.0 - Transparency;

	out_FragColor = FinalColor;
}
