/*
	Light - normalized surface-to-light vector
	Eye   - normalized surface-to-eye vector
*/
vec4 ComputeCookTorranceTerm( vec3 Light, vec3 Eye, vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 Half = normalize( Light + Eye );

	// Beckman's distribution function, D
	float NormalDotHalf = dot( Normal, Half );
	float NormalDotHalf2 = NormalDotHalf * NormalDotHalf;
	float Roughness = u_MaterialShading.z;
	// ...add an offset to avoid division by zero
	float Roughness2 = Roughness * Roughness + 0.00001;
	float Exponent = -(1.0 - NormalDotHalf2) / (NormalDotHalf2 * Roughness2);
	float e = 2.718281828459045;
	float D = pow(e, Exponent) / (Roughness2 * NormalDotHalf2 * NormalDotHalf2);

	// Compute Fresnel term, F
	float RefIndex = Specular.a; //texture2D(normalMap, vUV).a;
	float NormalDotEye = dot( Normal, Eye );
	float F = mix( pow(1.0 - NormalDotEye, 5.0), 1.0, RefIndex );

	// Compute self shadowing term, G
	float NormalDotLight = dot( Normal, Light );
	float X = 2.0 * NormalDotHalf / dot( Eye, Half );
	float G = min(1.0, min(X * NormalDotLight, X * NormalDotEye));

	// Compute final Cook-Torrance specular term, load textures, mix them
	float pi = 3.14159265358979;
	float CookTorrance = (D*F*G) / (NormalDotEye * pi);

	vec4 Kd = Diffuse  * max(0.0, NormalDotLight) * vec4(u_LightColor.xyz, 1.0);
	vec4 Ks = Specular * max(0.0, CookTorrance  ) * vec4(u_LightColor.xyz, 1.0);

	float ShineStrength = u_MaterialParams.z;

	return (Kd + Ks * ShineStrength);
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputeDirectionalLight_CookTorrance( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 CameraPos = in_ViewMatrixInverse[3].xyz;

	vec3 Light = normalize( -u_LightDirection.xyz );
	vec3 Eye   = normalize( CameraPos - v_WorldPosition );

	return ComputeCookTorranceTerm( Light, Eye, Normal, Diffuse, Specular );
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputePointLight_CookTorrance( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 CameraPos = in_ViewMatrixInverse[3].xyz;

	vec3 Light = normalize( u_LightPosition.xyz - v_WorldPosition );
	vec3 Eye   = normalize( CameraPos - v_WorldPosition );

	// attenuation
	float AttenuationCoef = AttenuationForLight( length( u_LightPosition.xyz - v_WorldPosition ) );

	return ComputeCookTorranceTerm( Light, Eye, Normal, Diffuse, Specular ) * AttenuationCoef;
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputeSpotLight_CookTorrance( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 LightDirection = normalize( v_WorldPosition - u_LightPosition.xyz );

	float InnerAngle = u_LightSpotRange.x;
	float OuterAngle = u_LightSpotRange.y;
	float Falloff    = u_LightSpotRange.z;

	float Rho = max( 0.0, dot( normalize( LightDirection ), u_LightDirection.xyz ) );

	vec4 Color = vec4( 0.0 );

	if ( Rho > OuterAngle )
	{
		float SpotFactor = pow( (Rho-OuterAngle) / (InnerAngle-OuterAngle), Falloff );
		Color = ComputePointLight_CookTorrance( Normal, Diffuse, Specular ) * SpotFactor;
	}

	return Color;
}
